import unittest

from algo.constant import Symbol

SYMBOL_STR = 'GBP_USD'


class TestSymbolOperations(unittest.TestCase):
    """ not as much as tests, more playground """

    def test_convert_from_str(self):
        self.assertEqual(Symbol(SYMBOL_STR), Symbol.GBP)

    def test_convert_from_str_back_to_str(self):
        self.assertEqual(Symbol(SYMBOL_STR).value, SYMBOL_STR)
